@@ -14,17 +14,24 @@ This is used on Debian and derivatives such as Ubuntu.
 
 import subprocess, os, glob, stat, sys, tempfile, shutil, time
 import hashlib
+import json
+
+from contextlib import closing
 
 import warnings
 warnings.filterwarnings('ignore', 'apt API not stable yet', FutureWarning)
 import apt
 try:
     import cPickle as pickle
-    from urllib import urlopen
-    (pickle, urlopen)  # pyflakes
+    from urllib import urlopen, quote, unquote
+    (pickle, urlopen, quote, unquote)  # pyflakes
+    URLError = IOError
+    HTTPError = IOError
 except ImportError:
     # python 3
+    from urllib.error import URLError, HTTPError
     from urllib.request import urlopen
+    from urllib.parse import quote, unquote
     import pickle
 
 import apport
@@ -41,6 +48,9 @@ class __AptDpkgPackageInfo(PackageInfo):
         self._contents_dir = None
         self._mirror = None
         self._virtual_mapping_obj = None
+        self._launchpad_base = 'https://api.launchpad.net/devel'
+        self._archive_url = self._launchpad_base + '/%s/main_archive'
+        self._ppa_archive_url = self._launchpad_base + '/~%(user)s/+archive/%(distro)s/%(ppaname)s'
 
     def __del__(self):
         try:
@@ -82,14 +92,15 @@ class __AptDpkgPackageInfo(PackageInfo):
                 self._apt_cache = apt.Cache(rootdir='/')
         return self._apt_cache
 
-    def _sandbox_cache(self, aptroot, apt_sources, fetchProgress):
+    def _sandbox_cache(self, aptroot, apt_sources, fetchProgress, distro_name, release_codename, origins):
         '''Build apt sandbox and return apt.Cache(rootdir=) (initialized lazily).
 
         Clear the package selection on subsequent calls.
         '''
         self._apt_cache = None
         if not self._sandbox_apt_cache:
-            self._build_apt_sandbox(aptroot, apt_sources)
+            self._build_apt_sandbox(aptroot, apt_sources, distro_name,
+                                    release_codename, origins)
             rootdir = os.path.abspath(aptroot)
             self._sandbox_apt_cache = apt.Cache(rootdir=rootdir)
             try:
@@ -175,7 +186,26 @@ class __AptDpkgPackageInfo(PackageInfo):
         if pkg.installed and pkg.installed.version is None:
             return False
 
-        native_origins = [self.get_os_version()[0]]
+        distro_name = self.get_os_version()[0]
+
+        if pkg.candidate and pkg.candidate.origins:  # might be None
+            for o in pkg.candidate.origins:
+                if o.origin == distro_name:
+                    return True
+        return False
+
+    def is_native_origin_package(self, package):
+        '''Check if a package originated from a native location
+
+        Return True for a package which came from an origin which is listed in
+        native-origins.d, False if it comes from a third-party source.
+        '''
+        pkg = self._apt_pkg(package)
+        # some PPA packages have installed version None, see LP#252734
+        if pkg.installed and pkg.installed.version is None:
+            return False
+
+        native_origins = []
         for f in glob.glob('/etc/apport/native-origins.d/*'):
             try:
                 with open(f) as fd:
@@ -192,10 +222,101 @@ class __AptDpkgPackageInfo(PackageInfo):
                     return True
         return False
 
+    def get_lp_binary_package(self, distro_id, package, version, arch):
+        package = quote(package)
+        version = quote(version)
+        ma = self.json_request(self._archive_url % distro_id)
+        if not ma:
+            return (None, None)
+        ma_link = ma['self_link']
+        pb_url = ma_link + ('/?ws.op=getPublishedBinaries&binary_name=%s&version=%s&exact_match=true' %
+                            (package, version))
+        bpub_url = ''
+        try:
+            pbs = self.json_request(pb_url, entries=True)
+            if not pbs:
+                return (None, None)
+            for pb in pbs:
+                if pb['architecture_specific'] == 'false':
+                    bpub_url = pb['self_link']
+                    break
+                else:
+                    if pb['distro_arch_series_link'].endswith(arch):
+                        bpub_url = pb['self_link']
+                        break
+        except IndexError:
+            return (None, None)
+        if not bpub_url:
+            return (None, None)
+        bf_urls = bpub_url + '?ws.op=binaryFileUrls&include_meta=true'
+        bfs = self.json_request(bf_urls)
+        if not bfs:
+            return (None, None)
+        for bf in bfs:
+            # return the first binary file url since there being more than one
+            # is theoretical
+            return (unquote(bf['url']), bf['sha1'])
+
+    def json_request(self, url, entries=False):
+        '''Open, read and parse the json of a url
+
+        Set entries to True when the json data returned by Launchpad
+        has a dictionary with an entries key which contains the data
+        desired.
+        '''
+        try:
+            response = urlopen(url)
+            if response.getcode() >= 400:
+                raise HTTPError('%u' % response.getcode())
+        except (URLError, HTTPError):
+            apport.warning('cannot connect to: %s' % unquote(url))
+            return None
+        try:
+            content = response.read()
+        except IOError:
+            apport.warning('failure reading data at: %s' % unquote(url))
+            return None
+        if isinstance(content, bytes):
+            content = content.decode('utf-8')
+        if entries:
+            return json.loads(content)['entries']
+        else:
+            return json.loads(content)
+
+    def get_lp_source_package(self, distro_id, package, version):
+        package = quote(package)
+        version = quote(version)
+        ma = self.json_request(self._archive_url % distro_id)
+        if not ma:
+            return None
+        ma_link = ma['self_link']
+        ps_url = ma_link + ('/?ws.op=getPublishedSources&exact_match=true&source_name=%s&version=%s' %
+                            (package, version))
+        # use the first entry as they are sorted chronologically
+        try:
+            ps = self.json_request(ps_url, entries=True)[0]['self_link']
+        except IndexError:
+            return None
+        if not ps:
+            return None
+        sf_urls = ps + '?ws.op=sourceFileUrls'
+        sfus = self.json_request(sf_urls)
+        if not sfus:
+            return None
+
+        source_files = []
+        for sfu in sfus:
+            if sys.version_info.major == 2 and isinstance(sfu, unicode):
+                    sfu = sfu.encode('utf-8')
+            sfu = unquote(sfu)
+            source_files.append(sfu)
+
+        return source_files
+
     def get_architecture(self, package):
         '''Return the architecture of a package.
 
-        This might differ on multiarch architectures (e. g.  an i386 Firefox
+        This might differ on multiarch architectures (e. g. an i386 Firefox
         package on a x86_64 system)'''
 
         if self._apt_pkg(package).installed:
@@ -450,7 +571,32 @@ Debug::NoLocking "true";
             argv[-1] += '=' + version
         try:
             if subprocess.call(argv, cwd=dir, env=env) != 0:
-                return None
+                if not version:
+                    return None
+                sf_urls = self.get_lp_source_package(self.get_distro_name(),
+                                                     srcpackage, version)
+                if sf_urls:
+                    proxy = ''
+                    if apt.apt_pkg.config.find('Acquire::http::Proxy') != '':
+                        proxy = apt.apt_pkg.config.find('Acquire::http::Proxy')
+                        apt.apt_pkg.config.set('Acquire::http::Proxy', '')
+                    fetchProgress = apt.progress.base.AcquireProgress()
+                    fetcher = apt.apt_pkg.Acquire(fetchProgress)
+                    af_queue = []
+                    for sf in sf_urls:
+                        af_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                        sf, destdir=dir))
+                    result = fetcher.run()
+                    if result != fetcher.RESULT_CONTINUE:
+                        return None
+                    if proxy:
+                        apt.apt_pkg.config.set('Acquire::http::Proxy', proxy)
+                    for dsc in glob.glob(os.path.join(dir, '*.dsc')):
+                        subprocess.call(['dpkg-source', '-sn',
+                                         '-x', dsc], stdout=subprocess.PIPE,
+                                        cwd=dir)
+                else:
+                    return None
         except OSError:
             return None
 
@@ -512,6 +658,7 @@ Debug::NoLocking "true";
                 break
             out.write(block)
         out.flush()
+        out.close()
         ret = subprocess.call(['dpkg', '-i', os.path.join(target_dir, deb)])
         if ret == 0:
             installed.append(deb.split('_')[0])
@@ -519,7 +666,8 @@ Debug::NoLocking "true";
 
     def install_packages(self, rootdir, configdir, release, packages,
                          verbose=False, cache_dir=None,
-                         permanent_rootdir=False, architecture=None):
+                         permanent_rootdir=False, architecture=None,
+                         origins=None):
         '''Install packages into a sandbox (for apport-retrace).
 
         In order to work without any special permissions and without touching
@@ -548,6 +696,9 @@ Debug::NoLocking "true";
         the given architecture (as specified in a report's "Architecture"
         field). If not given it defaults to the host system's architecture.
 
+        If origins is given, the sandbox will be created with apt data sources
+        for foreign origins.
+
         Return a string with outdated packages, or None if all packages were
         installed.
 
@@ -555,13 +706,15 @@ Debug::NoLocking "true";
         package servers down, etc.), this should raise a SystemError with a
         meaningful error message.
         '''
+        if not architecture:
+            architecture = self.get_system_architecture()
         if not configdir:
             apt_sources = '/etc/apt/sources.list'
             self.current_release_codename = self.get_distro_codename()
         else:
             # support architecture specific config, fall back to global config
             apt_sources = os.path.join(configdir, release, 'sources.list')
-            if architecture:
+            if architecture != self.get_system_architecture():
                 arch_apt_sources = os.path.join(configdir, release,
                                                 architecture, 'sources.list')
                 if os.path.exists(arch_apt_sources):
@@ -593,20 +746,25 @@ Debug::NoLocking "true";
             tmp_aptroot = True
             aptroot = tempfile.mkdtemp()
 
-        if architecture:
-            apt.apt_pkg.config.set('APT::Architecture', architecture)
-        else:
-            apt.apt_pkg.config.set('APT::Architecture', self.get_system_architecture())
+        apt.apt_pkg.config.set('APT::Architecture', architecture)
         apt.apt_pkg.config.set('Acquire::Languages', 'none')
+        # directly connect to Launchpad when downloading deb files
+        apt.apt_pkg.config.set('Acquire::http::Proxy::api.launchpad.net', 'direct')
+        apt.apt_pkg.config.set('Acquire::http::Proxy::launchpad.net', 'direct')
 
         if verbose:
             fetchProgress = apt.progress.text.AcquireProgress()
         else:
             fetchProgress = apt.progress.base.AcquireProgress()
         if not tmp_aptroot:
-            cache = self._sandbox_cache(aptroot, apt_sources, fetchProgress)
+            cache = self._sandbox_cache(aptroot, apt_sources, fetchProgress,
+                                        self.get_distro_name(),
+                                        self.current_release_codename,
+                                        origins)
         else:
-            self._build_apt_sandbox(aptroot, apt_sources)
+            self._build_apt_sandbox(aptroot, apt_sources,
+                                    self.get_distro_name(),
+                                    self.current_release_codename, origins)
             cache = apt.Cache(rootdir=os.path.abspath(aptroot))
             try:
                 cache.update(fetchProgress)
@@ -614,6 +772,8 @@ Debug::NoLocking "true";
                 raise SystemError(str(e))
             cache.open()
 
+        archivedir = apt.apt_pkg.config.find_dir("Dir::Cache::archives")
+
         obsolete = ''
 
         src_records = apt.apt_pkg.SourceRecords()
@@ -632,6 +792,10 @@ Debug::NoLocking "true";
 
         # mark packages for installation
         real_pkgs = set()
+        lp_cache = {}
+        fetcher = apt.apt_pkg.Acquire(fetchProgress)
+        # need to keep AcquireFile references
+        acquire_queue = []
         for (pkg, ver) in packages:
             try:
                 cache_pkg = cache[pkg]
@@ -646,7 +810,17 @@ Debug::NoLocking "true";
                 if ver:
                     cache_pkg.candidate = cache_pkg.versions[ver]
             except KeyError:
-                obsolete += '%s version %s required, but %s is available\n' % (pkg, ver, cache_pkg.candidate.version)
+                (lp_url, sha1sum) = self.get_lp_binary_package(self.get_distro_name(),
+                                                               pkg, ver, architecture)
+                if lp_url:
+                    acquire_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                                                 lp_url,
+                                                                 hash="sha1:%s" % sha1sum,
+                                                                 destdir=archivedir))
+                    lp_cache[pkg] = ver
+                else:
+                    obsolete += '%s version %s required, but %s is available\n' % (pkg, ver, cache_pkg.candidate.version)
+
             candidate = cache_pkg.candidate
             real_pkgs.add(pkg)
 
@@ -663,7 +837,6 @@ Debug::NoLocking "true";
                     conflicts += apt.apt_pkg.parse_depends(candidate.record['Conflicts'])
                 if 'Replaces' in candidate.record:
                     conflicts += apt.apt_pkg.parse_depends(candidate.record['Replaces'])
-                archives = apt.apt_pkg.config.find_dir('Dir::Cache::archives')
                 for conflict in conflicts:
                     # apt_pkg.parse_depends needs to handle the or operator,
                     # but as policy states it is invalid to use that in
@@ -679,7 +852,7 @@ Debug::NoLocking "true";
                             # unpacked into the sandbox.
                             continue
                         for p in providers:
-                            debs = os.path.join(archives, '%s_*.deb' % p)
+                            debs = os.path.join(archivedir, '%s_*.deb' % p)
                             for path in glob.glob(debs):
                                 ver = self._deb_version(path)
                                 if apt.apt_pkg.check_dep(ver, conflict[2], conflict[1]):
@@ -690,7 +863,7 @@ Debug::NoLocking "true";
                                 pass
                         del providers
                     else:
-                        debs = os.path.join(archives, '%s_*.deb' % conflict[0])
+                        debs = os.path.join(archivedir, '%s_*.deb' % conflict[0])
                         for path in glob.glob(debs):
                             ver = self._deb_version(path)
                             if apt.apt_pkg.check_dep(ver, conflict[2], conflict[1]):
@@ -702,53 +875,130 @@ Debug::NoLocking "true";
 
             if candidate.architecture != 'all':
                 try:
-                    dbg = cache[pkg + '-dbg']
+                    dbg_pkg = pkg + '-dbg'
+                    dbg = cache[dbg_pkg]
+                    pkg_found = False
                     # try to get the same version as pkg
-                    try:
-                        dbg.candidate = dbg.versions[candidate.version]
-                    except KeyError:
-                        obsolete += 'outdated -dbg package for %s: package version %s -dbg version %s\n' % (
-                            pkg, candidate.version, dbg.candidate.version)
-                    real_pkgs.add(pkg + '-dbg')
+                    if ver:
+                        try:
+                            dbg.candidate = dbg.versions[ver]
+                            pkg_found = True
+                        except KeyError:
+                            (lp_url, sha1sum) = self.get_lp_binary_package(self.get_distro_name(),
+                                                                           dbg_pkg, ver, architecture)
+                            if lp_url:
+                                acquire_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                                                             lp_url,
+                                                                             hash="sha1:%s" % sha1sum,
+                                                                             destdir=archivedir))
+                                lp_cache[dbg_pkg] = ver
+                                pkg_found = True
+                    if not pkg_found:
+                        try:
+                            dbg.candidate = dbg.versions[candidate.version]
+                        except KeyError:
+                            obsolete += 'outdated -dbg package for %s: package version %s -dbg version %s\n' % (
+                                pkg, ver, dbg.candidate.version)
+                    real_pkgs.add(dbg_pkg)
                 except KeyError:
                     # install all -dbg from the source package
                     if src_records.lookup(candidate.source_name):
-                        dbgs = [p for p in src_records.binaries if p.endswith('-dbg') and p in cache]
+                        # ignore transitional packages
+                        dbgs = [p for p in src_records.binaries
+                                if p.endswith('-dbg') and p in cache and
+                                'transitional' not in cache[p].candidate.description]
                     else:
                         dbgs = []
                     if dbgs:
                         for p in dbgs:
-                            # try to get the same version as pkg
-                            try:
-                                cache[p].candidate = cache[p].versions[candidate.version]
-                            except KeyError:
-                                # we don't really expect that, but it's possible that
-                                # other binaries have a different version
-                                pass
+                            # if the package has already been added to
+                            # real_pkgs don't search for it again
+                            if p in real_pkgs:
+                                continue
+                            pkg_found = False
+                            # prefer the version requested
+                            if ver:
+                                try:
+                                    cache[p].candidate = cache[p].versions[ver]
+                                    pkg_found = True
+                                except KeyError:
+                                    (lp_url, sha1sum) = self.get_lp_binary_package(self.get_distro_name(),
+                                                                                   p, ver, architecture)
+                                    if lp_url:
+                                        acquire_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                                                                     lp_url,
+                                                                                     hash="sha1:%s" % sha1sum,
+                                                                                     destdir=archivedir))
+                                        lp_cache[p] = ver
+                                        pkg_found = True
+                            if not pkg_found:
+                                try:
+                                    cache[p].candidate = cache[p].versions[candidate.version]
+                                except KeyError:
+                                    # we don't really expect that, but it's possible that
+                                    # other binaries have a different version
+                                    pass
                             real_pkgs.add(p)
                     else:
+                        pkg_found = False
+                        dbgsym_pkg = pkg + '-dbgsym'
                         try:
-                            dbgsym = cache[pkg + '-dbgsym']
-                            real_pkgs.add(pkg + '-dbgsym')
-                            try:
-                                dbgsym.candidate = dbgsym.versions[candidate.version]
-                            except KeyError:
-                                obsolete += 'outdated debug symbol package for %s: package version %s dbgsym version %s\n' % (
-                                    pkg, candidate.version, dbgsym.candidate.version)
+                            dbgsym = cache[dbgsym_pkg]
+                            real_pkgs.add(dbgsym_pkg)
+                            # prefer the version requested
+                            if ver:
+                                try:
+                                    dbgsym.candidate = dbgsym.versions[ver]
+                                    pkg_found = True
+                                except KeyError:
+                                    (lp_url, sha1sum) = self.get_lp_binary_package(self.get_distro_name(),
+                                                                                   dbgsym_pkg, ver, architecture)
+                                    if lp_url:
+                                        acquire_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                                                                     lp_url,
+                                                                                     hash="sha1:%s" % sha1sum,
+                                                                                     destdir=archivedir))
+                                        lp_cache[dbgsym_pkg] = ver
+                                        pkg_found = True
+                            if not pkg_found:
+                                try:
+                                    dbgsym.candidate = dbgsym.versions[candidate.version]
+                                except KeyError:
+                                    obsolete += 'outdated debug symbol package for %s: package version %s dbgsym version %s\n' % (
+                                        pkg, candidate.version, dbgsym.candidate.version)
                         except KeyError:
-                            obsolete += 'no debug symbol package found for %s\n' % pkg
+                            if ver:
+                                (lp_url, sha1sum) = self.get_lp_binary_package(self.get_distro_name(),
+                                                                               dbgsym_pkg, ver, architecture)
+                                if lp_url:
+                                    real_pkgs.add(dbgsym_pkg)
+                                    acquire_queue.append(apt.apt_pkg.AcquireFile(fetcher,
+                                                                                 lp_url,
+                                                                                 hash="sha1:%s" % sha1sum,
+                                                                                 destdir=archivedir))
+                                    lp_cache[dbgsym_pkg] = ver
+                                    pkg_found = True
+                            if not pkg_found:
+                                obsolete += 'no debug symbol package found for %s\n' % pkg
 
         # unpack packages, weed out the ones that are already installed (for
         # permanent sandboxes)
         for p in real_pkgs.copy():
-            if pkg_versions.get(p) != cache[p].candidate.version:
-                cache[p].mark_install(False, False)
+            if ver:
+                if pkg_versions.get(p) != ver:
+                    cache[p].mark_install(False, False)
+                elif pkg_versions.get(p) != cache[p].candidate.version:
+                    cache[p].mark_install(False, False)
+                else:
+                    real_pkgs.remove(p)
             else:
-                real_pkgs.remove(p)
+                if pkg_versions.get(p) != cache[p].candidate.version:
+                    cache[p].mark_install(False, False)
+                else:
+                    real_pkgs.remove(p)
 
         last_written = time.time()
         # fetch packages
-        fetcher = apt.apt_pkg.Acquire(fetchProgress)
         try:
             cache.fetch_archives(fetcher=fetcher)
         except apt.cache.FetchFailedException as e:
@@ -761,9 +1011,22 @@ Debug::NoLocking "true";
             if not permanent_rootdir or os.path.getctime(i.destfile) > last_written:
                 out = subprocess.check_output(['dpkg-deb', '--show', i.destfile]).decode()
                 (p, v) = out.strip().split()
-                pkg_versions[p] = v
-                subprocess.check_call(['dpkg', '-x', i.destfile, rootdir])
-            real_pkgs.remove(os.path.basename(i.destfile).split('_', 1)[0])
+                # don't extract the same version of the package if it is
+                # already extracted
+                if pkg_versions.get(p) == v:
+                    pass
+                # don't extract the package if it is a different version than
+                # the one we want to extract from Launchpad
+                elif p in lp_cache and lp_cache[p] != v:
+                    pass
+                else:
+                    subprocess.check_call(['dpkg', '-x', i.destfile, rootdir])
+                    pkg_versions[p] = v
+            pkg_name = os.path.basename(i.destfile).split('_', 1)[0]
+            # because a package may exist multiple times in the fetcher it may
+            # have already been removed
+            if pkg_name in real_pkgs:
+                real_pkgs.remove(pkg_name)
 
         # update package list
         pkgs = list(pkg_versions.keys())
@@ -967,7 +1230,73 @@ Debug::NoLocking "true";
         return None
 
     @classmethod
-    def _build_apt_sandbox(klass, apt_root, apt_sources):
+    def create_ppa_source_from_origin(klass, origin, distro, release_codename):
+        '''For an origin from a Launchpad PPA create sources.list content.
+
+        distro is the distribution for which content is being created e.g.
+        ubuntu.
+
+        release_codename is the codename of the release for which content is
+        being created e.g. trusty.
+
+        Return a string containing content suitable for writing to a sources.list
+        file, or None if the origin is not a Launchpad PPA.
+        '''
+
+        if origin.startswith("LP-PPA-"):
+            components = origin.split("-")[2:]
+            # If the PPA is unnamed, it will not appear in origin information
+            # but is named ppa in Launchpad.
+            try_ppa = True
+            if len(components) == 1:
+                components.append('ppa')
+                try_ppa = False
+
+            index = 1
+            while index < len(components):
+                # For an origin we can't tell where the user name ends and the
+                # PPA name starts, so split on each "-" until we find a PPA
+                # that exists.
+                user = str.join('-', components[:index])
+                ppa_name = str.join('-', components[index:])
+                try:
+                    with closing(urlopen(apport.packaging._ppa_archive_url %
+                                         {'user': user, 'distro': distro,
+                                          'ppaname': ppa_name})) as response:
+                        response.read()
+                        # in Python 2 an error does not raise an exception
+                        if response.getcode() >= 400:
+                            raise HTTPError('%u' % response.getcode())
+                except (URLError, HTTPError):
+                    index += 1
+                    if index == len(components):
+                        if try_ppa:
+                            components.append('ppa')
+                            try_ppa = False
+                            index = 2
+                        else:
+                            user = None
+                    continue
+                break
+            if user and ppa_name:
+                ppa_line = 'deb http://ppa.launchpad.net/%s/%s/%s %s main' % \
+                           (user, ppa_name, distro, release_codename)
+                debug_url = 'http://ppa.launchpad.net/%s/%s/%s/dists/%s/main/debug' % \
+                            (user, ppa_name, distro, release_codename)
+                try:
+                    with closing(urlopen(debug_url)) as response:
+                        response.read()
+                        # in Python 2 an error does not raise an exception
+                        if response.getcode() >= 400:
+                            raise HTTPError('%u' % response.getcode())
+                    add_debug = ' main/debug'
+                except (URLError, HTTPError):
+                    add_debug = ''
+                return ppa_line + add_debug + '\ndeb-src' + ppa_line[3:] + '\n'
+        return None
+
+    @classmethod
+    def _build_apt_sandbox(klass, apt_root, apt_sources, distro_name, release_codename, origins):
         # pre-create directories, to avoid apt.Cache() printing "creating..."
         # messages on stdout
         if not os.path.exists(os.path.join(apt_root, 'var', 'lib', 'apt')):
@@ -989,6 +1318,46 @@ Debug::NoLocking "true";
             with open(os.path.join(apt_root, 'etc', 'apt', 'sources.list'), 'w') as dest:
                 dest.write(src.read())
 
+        if origins:
+            source_list_content = ''
+            # map an origin to a Launchpad username and PPA name
+            origin_data = {}
+            for origin in origins:
+                # apport's report format uses unknown for packages w/o an origin
+                if origin == 'unknown':
+                    continue
+                origin_path = None
+                if os.path.isdir(apt_sources + '.d'):
+                    # check to see if there is a sources.list file for the origin,
+                    # if there isn't try using a sources.list file w/o LP-PPA-
+                    origin_path = os.path.join(apt_sources + '.d', origin + '.list')
+                    if not os.path.exists(origin_path) and 'LP-PPA' in origin:
+                        origin_path = os.path.join(apt_sources + '.d',
+                                                   origin.strip('LP-PPA-') + '.list')
+                        if not os.path.exists(origin_path):
+                            origin_path = None
+                    elif not os.path.exists(origin_path):
+                        origin_path = None
+                if origin_path:
+                    with open(origin_path) as src_ext:
+                        source_list_content = src_ext.read()
+                else:
+                    source_list_content = klass.create_ppa_source_from_origin(origin, distro_name, release_codename)
+                if source_list_content:
+                    with open(os.path.join(apt_root, 'etc', 'apt',
+                                           'sources.list.d', origin + '.list'), 'a') as dest:
+                        dest.write(source_list_content)
+                    for line in source_list_content.splitlines():
+                        if line.startswith('#'):
+                            continue
+                        if 'ppa.launchpad.net' not in line:
+                            continue
+                        user = line.split()[1].split('/')[3]
+                        ppa = line.split()[1].split('/')[4]
+                        origin_data[origin] = (user, ppa)
+                else:
+                    apport.warning("Could not find or create source config for %s" % origin)
+
         # install apt keyrings; prefer the ones from the config dir, fall back
         # to system
         trusted_gpg = os.path.join(os.path.dirname(apt_sources), 'trusted.gpg')
@@ -1008,12 +1377,42 @@ Debug::NoLocking "true";
         else:
             os.makedirs(trusted_d)
 
+        # install apt keyrings for PPAs
+        if origins and source_list_content:
+            for origin, (ppa_user, ppa_name) in origin_data.items():
+                ppa_archive_url = apport.packaging._ppa_archive_url % \
+                    {'user': quote(ppa_user), 'distro': distro_name,
+                     'ppaname': quote(ppa_name)}
+                ppa_info = apport.packaging.json_request(ppa_archive_url)
+                if not ppa_info:
+                    continue
+                try:
+                    signing_key_fingerprint = ppa_info['signing_key_fingerprint']
+                except IndexError:
+                    apport.warning("Error: can't find signing_key_fingerprint at %s"
+                                   % ppa_archive_url)
+                    continue
+                argv = ['gpg', '--no-options',
+                        '--no-default-keyring',
+                        '--no-auto-check-trustdb',
+                        '--keyring',
+                        os.path.join(trusted_d,
+                                     '%s.gpg' % origin),
+                        ]
+                argv += ['--quiet', '--batch',
+                         '--keyserver', 'hkp://keyserver.ubuntu.com:80/',
+                         '--recv', signing_key_fingerprint]
+                if subprocess.call(argv) != 0:
+                    apport.warning('Unable to import key for %s' %
+                                   ppa_archive_url)
+                    pass
+
     @classmethod
     def _deb_version(klass, pkg):
         '''Return the version of a .deb file'''
 
         dpkg = subprocess.Popen(['dpkg-deb', '-f', pkg, 'Version'], stdout=subprocess.PIPE)
-        out = dpkg.communicate(input)[0].decode('UTF-8')
+        out = dpkg.communicate(input)[0].decode('UTF-8').strip()
         assert dpkg.returncode == 0
         assert out
         return out
@@ -1038,4 +1437,18 @@ Debug::NoLocking "true";
 
         return self._distro_codename
 
+    _distro_name = None
+
+    def get_distro_name(self):
+        '''Get osname from /etc/os-release, or if that doesn't exist,
+           'lsb_release -sir' output and cache the result.'''
+
+        if self._distro_name is None:
+            self._distro_name = self.get_os_version()[0].lower()
+            if ' ' in self._distro_name:
+                # concatenate distro name e.g. ubuntu-rtm
+                self._distro_name = self._distro_name.replace(' ', '-')
+
+        return self._distro_name
+
 impl = __AptDpkgPackageInfo()
