@@ -119,7 +119,7 @@ class ProblemReport(UserDict):
 
         file needs to be opened in binary mode.
 
-        Files are in RFC822 format.
+        Files are in RFC822 format, but with case sensitive keys.
         '''
         self._assert_bin_mode(file)
         self.data.clear()
@@ -629,8 +629,8 @@ class ProblemReport(UserDict):
         # (tuple (string|file [, bool]))
         assert (isinstance(v, CompressedValue) or hasattr(v, 'isalnum') or
                 (hasattr(v, '__getitem__') and (
-                    len(v) == 1 or (len(v) >= 2 and v[1] in (True, False)))
-                    and (hasattr(v[0], 'isalnum') or hasattr(v[0], 'read'))))
+                    len(v) == 1 or (len(v) >= 2 and v[1] in (True, False))) and
+                    (hasattr(v[0], 'isalnum') or hasattr(v[0], 'read'))))
 
         return self.data.__setitem__(k, v)
 
